require 'mysql2'
require File.expand_path("../../../config/environment", __FILE__)
require_relative 'base'

class GossamerForumsImporter < ImportScripts::Base
  def initialize
    super
    # Initialize MySQL client with connection details
    @mysql_client = Mysql2::Client.new(
      host: "slowtwitch.northend.network",
      username: "admin",
      password: "yxnh93Ybbz2Nm8#mp28zCVv",
      database: "slowtwitch"
    )
  end

  # Execute an SQL query on the Gossamer Forums MySQL database
  def execute_query(query)
    @mysql_client.query(query, as: :hash)
  end

  # Sanitize username to comply with Discourse's rules
  def sanitize_username(username, email, name)
    original_username = username
    # Replace unacceptable characters with underscores
    sanitized = username.gsub(/[^a-zA-Z0-9._-]/, '_')
    # Ensure the username is at least 2 characters long
    sanitized = "#{sanitized}." if sanitized.length < 2
    # Ensure the username is no more than 20 characters long
    sanitized = sanitized[0, 20] if sanitized.length > 20
    original_sanitized = sanitized

    # Check for existing user with the same username
    existing_user = User.find_by(username: sanitized)
    
    if existing_user
      # If email and name match, do not modify the username
#      puts "existing_user.email: '#{existing_user.email}'  vs.  email: '#{email}'"
#      puts "existing_user.name: '#{existing_user.name}'  vs.  email: '#{name}'"
      if existing_user.email.downcase == email.downcase && existing_user.name == name
        return sanitized
      else
        # Ensure the username is unique
        counter = 1
        while User.exists?(username: sanitized)
          sanitized = "#{original_sanitized}_#{counter}"
          sanitized = sanitized[0, 20] if sanitized.length > 20
          counter += 1
        end
      end
    end

    # Print the original and sanitized usernames if they differ
    if original_username != sanitized
      puts "Sanitized username: '#{original_username}' --> '#{sanitized}'"
#    else
#      puts "UNsanitized username:  '#{original_username}' --> '#{sanitized}'"
    end

    sanitized
  end

  # Sanitize email to replace restricted domains
  def sanitize_email(email)
    restricted_domains = ['mailinator.com', 'example.com'] # Add more restricted domains as needed
    domain = email.split('@').last

    if restricted_domains.include?(domain)
      sanitized_email = email.gsub(domain, 'example.org') # Change to a permissible domain
      puts "Sanitized email: '#{email}' --> '#{sanitized_email}'"
      return sanitized_email
    end

    email
  end

  # Import users from Gossamer Forums gforum_User table to Discourse
  def import_users
    puts "Importing users..."
    users = []

    # Fetch all users from Gossamer Forums
    execute_query("SELECT * FROM gforum_User").each do |row|

#      sanitized_username = sanitize_username(row['user_username'], row['user_email'], row['user_real_name'])
#      sanitized_email = sanitize_email(row['user_email'])

      users << {
        id: row['user_id'],
        username: sanitize_username(row['user_username'], row['user_email'], row['user_real_name']),
        email: row['user_email'],
        created_at: Time.at(row['user_registered']),
        updated_at: Time.at(row['user_last_seen']),
        name: row['user_real_name'],
        title: row['user_title'],
        bio_raw: row['user_about'] || "",
        website: row['user_homepage'],
        location: row['user_location'],
        custom_fields: {
          md5_password: row['user_password'],
          original_username: row['user_username'],
          original_gossamer_id: row['user_id']
        }
      }
    end

    # Create or update users in Discourse
    create_users(users) do |user|
      user
    end

    # Append user bio and import user files
    users.each do |user|
      discourse_username = sanitize_username(user[:username], user[:email], user[:name])
      discourse_user = User.find_by(username: discourse_username)

      if discourse_user.nil?
        puts "User #{user[:username]} --> #{discourse_username} not found in Discourse. Skipping file import."
        next
      end

      if discourse_user.user_profile.bio_raw.nil? || discourse_user.user_profile.bio_raw.empty?
        discourse_user.user_profile.bio_raw = user[:bio_raw]
      else
        discourse_user.user_profile.bio_raw += "\n\n" + user[:bio_raw]
      end

      if discourse_user.user_profile.bio_raw.length > 2999
        puts "Warning: About Me for user #{discourse_user.username} (ID: #{discourse_user.id}) exceeds 3000 characters. Truncating."
        discourse_user.user_profile.bio_raw = discourse_user.user_profile.bio_raw[0, 2999]
      end
      discourse_user.user_profile.save!

      import_user_files(discourse_user)
    end
  end

  # Import user files from Gossamer Forums to Discourse
  def import_user_files(user)
    print "\rImporting files for user #{user.username}..."

    original_gossamer_id = user.custom_fields['original_gossamer_id']
    if original_gossamer_id.nil? || original_gossamer_id.empty?
      puts "User #{user.username} does not have a valid original_gossamer_id. Skipping file import."
      return
    end

#    puts "Original Gossamer ID for user #{user.username}: #{original_gossamer_id}"

    execute_query("SELECT * FROM gforum_User_Files WHERE ForeignColKey = #{original_gossamer_id}").each do |file|
      # Construct the file URL
      file_url = "https://forum.slowtwitch.com/images/users/images/#{file['ID'] % 10}/#{file['ID']}-#{file['File_Name']}"
      puts "User #{user.username}  User ID: #{user.id}  original_gossamer_id: #{original_gossamer_id}  file_url: #{file_url}"
      
      new_bio = user.user_profile.bio_raw + "\n\n![#{file['File_Name']}](#{file_url})"
      if new_bio.length > 3000
        puts "Warning: About Me for user #{user.username} (ID: #{user.id}) exceeds 3000 characters after adding file link. Truncating."
        new_bio = new_bio[0, 3000]
      end
      user.user_profile.bio_raw = new_bio
      user.user_profile.save!
    end
    print "Importing files for user #{user.username}... Done.\n"
  end

  # Import categories from Gossamer Forums to Discourse
  def import_categories
    puts "Importing categories..."
    execute_query("SELECT * FROM gforum_Category").each do |row|
      # Only create category if it does not exist
      unless CategoryCustomField.exists?(name: 'original_gossamer_id', value: row['category_id'])
        category = create_category(
          id: row['category_id'],
          name: row['name'],
          description: row['description'],
          created_at: row['created_at'] ? Time.at(row['created_at']) : Time.now,
          updated_at: row['updated_at'] ? Time.at(row['updated_at']) : Time.now
        )
        category.custom_fields.create!(name: 'original_gossamer_id', value: row['category_id'])
      end
    end
  end

  # Import topics and posts from Gossamer Forums to Discourse
  def import_topics_and_posts
    puts "Importing topics and posts..."
    execute_query("SELECT * FROM gforum_Post ORDER BY post_root_id, post_time").each do |row|
      if row['post_id'] == row['post_root_id']
        # Skip if the topic already exists
        unless TopicCustomField.exists?(name: 'original_gossamer_id', value: row['post_id'])
          # Create the topic
          topic = create_topic(
            id: row['post_id'],
            title: row['post_subject'],
            user_id: row['user_id_fk'],
            created_at: Time.at(row['post_time']),
            updated_at: Time.at(row['post_latest_reply']),
            category_id: row['forum_id_fk']
          )
          topic.custom_fields.create!(name: 'original_gossamer_id', value: row['post_id'])

          # Create the initial post in the topic
          post = create_post(
            id: row['post_id'],
            topic_id: row['post_id'],
            user_id: row['user_id_fk'],
            raw: import_post_attachments(row['post_message'], row['post_id']),
            created_at: Time.at(row['post_time']),
            updated_at: Time.at(row['post_latest_reply'])
          )
          post.custom_fields.create!(name: 'original_gossamer_id', value: row['post_id'])
        end
      else
        # Skip if the post already exists
        unless PostCustomField.exists?(name: 'original_gossamer_id', value: row['post_id'])
          # Create the post in the existing topic
          post = create_post(
            id: row['post_id'],
            topic_id: row['post_root_id'],
            user_id: row['user_id_fk'],
            raw: import_post_attachments(row['post_message'], row['post_id']),
            created_at: Time.at(row['post_time']),
            updated_at: Time.at(row['post_latest_reply']),
            reply_to_post_number: row['post_father_id']
          )
          post.custom_fields.create!(name: 'original_gossamer_id', value: row['post_id'])
        end
      end
    end
  end

  # Import attachments for a post
  def import_post_attachments(post_message, post_id)
    attachments = execute_query("SELECT * FROM gforum_PostAttachment WHERE post_id_fk = #{post_id}")
    attachments.each do |attachment|
      attachment_url = "https://forum.slowtwitch.com/images/posts/#{post_id}/#{attachment['attachment_name']}"
      post_message += "\n\n![#{attachment['attachment_name']}](#{attachment_url})"
    end
    post_message
  end

  # Import personal messages (both inbox and sent messages)
  def import_personal_messages
    puts "Importing personal messages..."
    import_inbox_messages
    import_sent_messages
  end

  # Import inbox messages from gforum_Message table
  def import_inbox_messages
    puts "Importing inbox messages..."
    execute_query("SELECT * FROM gforum_Message").each do |row|
      unless TopicCustomField.exists?(name: 'original_gossamer_msg_id', value: row['msg_id'])
        # Create a private message topic in Discourse
        topic = create_topic(
          title: row['msg_subject'],
          user_id: row['from_user_id_fk'],
          archetype: Archetype.private_message,
          created_at: Time.at(row['msg_time']),
          updated_at: Time.at(row['msg_time'])
        )
        topic.custom_fields.create!(name: 'original_gossamer_msg_id', value: row['msg_id'])

        # Create the message as a post in the private topic
        create_post(
          topic_id: topic.id,
          user_id: row['from_user_id_fk'],
          raw: row['msg_body'],
          created_at: Time.at(row['msg_time']),
          updated_at: Time.at(row['msg_time'])
        )

        # Add recipient user to the private message topic
        topic.topic_allowed_users.create!(user_id: row['to_user_id_fk'])
      end
    end
  end

  # Import sent messages from gforum_SentMessage table
  def import_sent_messages
    puts "Importing sent messages..."
    execute_query("SELECT * FROM gforum_SentMessage").each do |row|
      unless TopicCustomField.exists?(name: 'original_gossamer_sent_msg_id', value: row['msg_id'])
        # Create a private message topic in Discourse
        topic = create_topic(
          title: row['msg_subject'],
          user_id: row['from_user_id_fk'],
          archetype: Archetype.private_message,
          created_at: Time.at(row['msg_time']),
          updated_at: Time.at(row['msg_time'])
        )
        topic.custom_fields.create!(name: 'original_gossamer_sent_msg_id', value: row['msg_id'])

        # Create the message as a post in the private topic
        create_post(
          topic_id: topic.id,
          user_id: row['from_user_id_fk'],
          raw: row['msg_body'],
          created_at: Time.at(row['msg_time']),
          updated_at: Time.at(row['msg_time'])
        )

        # Add recipient user to the private message topic
        topic.topic_allowed_users.create!(user_id: row['to_user_id_fk'])
      end
    end
  end

  # Main method to perform the import
  def perform_import
    puts "Starting Gossamer Forums import..."
    import_users
    import_categories
    import_topics_and_posts
    import_personal_messages
    puts "Gossamer Forums import complete!"
  end
end

# Create an instance of the importer and start the import process
# importer = GossamerForumsImporter.new
# importer.perform_import

GossamerForumsImporter.new.perform_import

